import influxdb_client
from flask import Flask, current_app, g

_no_influx_msg = """\
InfluxDB connection is not present.
This means that something has overwritten g.influxdb.
"""


class InfluxDB(object):
    def __init__(self, app: Flask = None) -> None:
        """
        Constructor Class for InfluxDB
        :param app: Flask application
        """
        self.app = app
        if app is not None:
            self.init_app(app)

    def init_app(self, app: Flask) -> None:
        """
        Initialize influxdb for application
        :param app: Flask Application object
        """
        app.config.setdefault("INFLUXDB_V2_URL", "http://localhost:8086")
        app.config.setdefault("INFLUXDB_V2_ORG", None)
        app.config.setdefault("INFLUXDB_V2_TOKEN", None)
        app.config.setdefault("INFLUXDB_V2_TIMEOUT", 10000)
        app.config.setdefault("INFLUXDB_V2_VERIFY_SSL", False)
        app.config.setdefault("INFLUXDB_V2_SSL_CA_CERT", None)
        app.config.setdefault("INFLUXDB_V2_CONNECTION_POOL_MAXSIZE", 10)
        app.config.setdefault("INFLUXDB_V2_AUTH_BASIC", False)

        app.teardown_appcontext(self.teardown)

    @staticmethod
    def teardown(exception) -> None:
        """
        Method for tearing down influxdb input
        """
        if hasattr(g, "influxdb") and g.influxdb is not None:
            g.influxdb.close()

    @staticmethod
    def connect() -> influxdb_client.InfluxDBClient:
        """
        Connect to influxdb v2 using configuration parameters.
        :return: InfluxDBClient object
        """
        return influxdb_client.InfluxDBClient(
            url=current_app.config["INFLUXDB_V2_URL"],
            org=current_app.config["INFLUXDB_V2_ORG"],
            token=current_app.config["INFLUXDB_V2_TOKEN"],
            timeout=current_app.config["INFLUXDB_V2_TIMEOUT"],
            verify_ssl=current_app.config["INFLUXDB_V2_VERIFY_SSL"],
            ssl_ca_cert=current_app.config["INFLUXDB_V2_SSL_CA_CERT"],
            connection_pool_maxsize=current_app.config[
                "INFLUXDB_V2_CONNECTION_POOL_MAXSIZE"
            ],
            auth_basic=current_app.config["INFLUXDB_V2_AUTH_BASIC"],
        )

    @property
    def connection(self) -> influxdb_client.InfluxDBClient:
        """
        InfluxDBClient object
        :return:
        """
        if not hasattr(g, "influxdb"):
            g.influxdb = self.connect()
        if g.influxdb is None:
            raise RuntimeError(_no_influx_msg)

        return g.influxdb

    @property
    def close(self) -> callable:
        return self.connection.close

    @property
    def buckets_api(self) -> callable:
        return self.connection.buckets_api

    @property
    def authorizations_api(self) -> callable:
        return self.connection.authorizations_api

    @property
    def delete_api(self) -> callable:
        return self.connection.delete_api

    @property
    def health(self) -> callable:
        return self.connection.health

    @property
    def labels_api(self) -> callable:
        return self.connection.labels_api

    @property
    def organizations_api(self) -> callable:
        return self.connection.organizations_api

    @property
    def ping(self) -> callable:
        return self.connection.ping

    @property
    def query_api(self) -> callable:
        return self.connection.query_api

    @property
    def ready(self) -> callable:
        return self.connection.ready

    @property
    def tasks_api(self) -> callable:
        return self.connection.tasks_api

    @property
    def users_api(self) -> callable:
        return self.connection.users_api

    @property
    def version(self) -> callable:
        return self.connection.version

    @property
    def write_api(self) -> callable:
        return self.connection.write_api



import os
from collections import namedtuple
from datetime import datetime

import pytest
from flask import Flask, jsonify
from influxdb_client.client.write_api import SYNCHRONOUS

from influxdb_flask import InfluxDB

App = namedtuple("App", ["ctx", "client"])

influxdb = InfluxDB()


def create_app(config: str) -> Flask:
    app = Flask(__name__)
    app.config.from_pyfile(config)

    influxdb.init_app(app)

    @app.route("/add/<location>/<value>")
    def write(location: str, value: int):
        point = {
            "measurement": "temperature",
            "tags": {"location": location},
            "fields": {"value": value},
            "time": datetime.utcnow(),
        }
        write_api = influxdb.write_api(SYNCHRONOUS)
        result = write_api.write(
            app.config["INFLUXDB_V2_BUCKET"], app.config["INFLUXDB_V2_ORG"], point
        )
        return jsonify(result=result)

    return app


@pytest.fixture(scope="session")
def influx():
    return influxdb


@pytest.fixture()
def app(request):
    """Session wide test application"""
    rel_dir = os.path.dirname(__file__)
    config = os.path.join(rel_dir, "config.cfg")

    app = create_app(config)

    client = app.test_client()
    ctx = app.app_context()

    yield App(ctx, client)


@pytest.fixture()
def app_no_cleanup(request):
    """Session wide test application"""
    rel_dir = os.path.dirname(__file__)
    config = os.path.join(rel_dir, "config.cfg")

    # Create app
    app = create_app(config)

    client = app.test_client()
    ctx = app.app_context()

    yield App(ctx, client)



import os
from datetime import datetime

import influxdb_client
import pytest
from flask import Flask, g
from influxdb_client.client.write_api import SYNCHRONOUS

from influxdb_flask import __version__
from influxdb_flask.influxdb_flask import InfluxDB, _no_influx_msg

_no_app_msg = """\
Working outside of application context.

This typically means that you attempted to use functionality that needed
the current application. To solve this, set up an application context
with app.app_context(). See the documentation for more information.\
"""


def test_version():
    assert __version__ == "0.1.2"


class TestBase:
    def test_outside_ctx(self, influx):
        with pytest.raises(RuntimeError) as excinfo:
            influx.query_api().query('from(bucket:"test") |> range(start: -10m)')

        assert str(excinfo.value) == _no_app_msg

    def test_overwrite_influxdb(self, app_no_cleanup, influx):
        app = app_no_cleanup

        with app.ctx, pytest.raises(RuntimeError) as excinfo:
            g.influxdb = None
            influx.query_api().query('from(bucket:"test") |> range(start: -10m)')

        assert str(excinfo.value) == _no_influx_msg

    def test_instance(self, app, influx):
        with app.ctx:
            influx.query_api().query('from(bucket:"test") |> range(start: -10m)')

            flux = g.influxdb
        assert isinstance(flux, influxdb_client.InfluxDBClient)

    def test_app_non_factory_pattern(self):
        app = Flask(__name__)
        influx = InfluxDB(app)

        rel_dir = os.path.dirname(__file__)
        config = os.path.join(rel_dir, "config.cfg")

        app.config.from_pyfile(config)

        with app.app_context():
            assert isinstance(influx.connect(), influxdb_client.InfluxDBClient)


class TestAPI:
    def test_base(self, app, influx):
        with app.ctx:
            assert hasattr(influx, "connection")
            assert hasattr(influx, "close")
            assert hasattr(influx, "buckets_api")
            assert hasattr(influx, "authorizations_api")
            assert hasattr(influx, "delete_api")
            assert hasattr(influx, "health")
            assert hasattr(influx, "labels_api")
            assert hasattr(influx, "organizations_api")
            assert hasattr(influx, "ping")
            assert hasattr(influx, "query_api")
            assert hasattr(influx, "ready")
            assert hasattr(influx, "tasks_api")
            assert hasattr(influx, "users_api")
            assert hasattr(influx, "version")
            assert hasattr(influx, "write_api")


class TestTags:
    def test_values(self, app, influx):
        measurement = "test_tags"
        points = [
            {
                "fields": {"index": 1},
                "tags": {"info": "test1"},
                "measurement": measurement,
                "time": datetime.utcnow(),
            },
            {
                "fields": {"index": 2},
                "tags": {"info": "test2"},
                "measurement": measurement,
                "time": datetime.utcnow(),
            },
        ]

        with app.ctx:
            influx.write_api(SYNCHRONOUS).write("test", "test", points[0])
            influx.write_api(SYNCHRONOUS).write("test", "test", points[1])
            result = influx.query_api().query(
                (
                    'from(bucket: "test") |> range(start: -10m) |> filter(fn: (r) => r["info"] == "test1" or'
                    ' r["info"] == "test2") |> aggregateWindow(every: 1m, fn: last, createEmpty: false) |> '
                    'yield(name: "last")'
                )
            )
        assert isinstance(result, list)

        result = list(result)
        assert len(result) == 2
        assert "test1" in result[0].records[0].values["info"]
        assert "test2" in result[1].records[0].values["info"]

    def test_keys(self, app, influx):
        measurement = "test_tags"
        points = [
            {
                "fields": {"index": 1},
                "tags": {"info": "test1"},
                "measurement": measurement,
                "time": datetime.utcnow(),
            },
            {
                "fields": {"index": 2},
                "tags": {"info": "test2"},
                "measurement": measurement,
                "time": datetime.utcnow(),
            },
        ]

        with app.ctx:
            influx.write_api(SYNCHRONOUS).write("test", "test", points[0])
            influx.write_api(SYNCHRONOUS).write("test", "test", points[1])
            result = influx.query_api().query(
                (
                    'from(bucket: "test") |> range(start: -10m) |> '
                    'filter(fn: (r) => r["_measurement"] == "test_tags") |> '
                    'aggregateWindow(every: 1m, fn: last, createEmpty: false) |> yield(name: "last")'
                )
            )
        assert isinstance(result, list)
        assert "info" in result[0].records[0].values
        assert "info" in result[1].records[0].values


class TestApp:
    def test_response(self, app, influx):
        result = app.client.get("/add/dhaka/30")
        assert result.status_code == 200

        with app.ctx:
            result = influx.query_api().query(
                (
                    'from(bucket: "test") |> range(start: -10m) |> '
                    'filter(fn: (r) => r["_measurement"] == "temperature")'
                    ' |> aggregateWindow(every: 1m, fn: last, createEmpty: false) |> yield(name: "last")'
                )
            )
        assert len(result) == 1
        assert result[0].records[0].values["location"] == "dhaka"
        assert result[0].records[0].values["_value"] == "30"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['influxdb_flask']

package_data = \
{'': ['*']}

install_requires = \
['Flask>=2.0.3,<3.0.0', 'influxdb-client>=1.25.0,<2.0.0']

setup_kwargs = {
    'name': 'influxdb-flask',
    'version': '0.1.2',
    'description': 'InfluxDB-Flask adds influxdb-client-python support to Flask.',
    'long_description': '\n# InfluxDB-Flask\n\nDocumentation: [InfluxDB-Flask Documentation](https://influxdb-flask.readthedocs.io/en/latest/)\n## Introduction\n\nInfluxDB went for a massive change from v1.7 to v2.0. They introduced a\nnew client library\n[influxdb-client-python](https://github.com/influxdata/influxdb-client-python)\nto interact with the InfluxDB v2 API. InfluxDB-Flask adds\ninfluxdb-client-python support to Flask.\n\n**Note: Use this library with InfluxDB 2.x and InfluxDB 1.8+.** For\nconnecting to **InfluxDB 1.7** or earlier instances, use the\n[Flask-InfluxDB](https://github.com/btashton/flask-influxdb) that uses\n[InfluxDB-Python](https://github.com/influxdata/influxdb-python) as\nclient library.\n\nThe API of the **influxdb-client-python** is not the\nbackwards-compatible with the old one - **influxdb-python.**\n\n## Installation\n\nInstall the extension via pip:\n\n    $ pip install influxdb-flask\n\n## Set Up\n\nInfluxdb_client can be accessed via InfluxDB class:\n\n    from flask import Flask\n    from influxdb_flask import InfluxDB\n\n    app = Flask(__name__)\n    influxdb = InfluxDB(app)\n\nDelayed configuration of `InfluxDB` is also supported via **init_app**\nmethod:\n\n    influxdb = InfluxDB()\n\n    app = Flask(__name__)\n    influxdb.init_app()\n\nCurrently `InfluxDB.connection` instance provides the functionality of\n`InfluxDBClient` .\n\nAn included example shows how to write and query data from InfluxDB.\n\n## Configuring InfluxDB-Flask\n\nThe following configuration values can be set for InfluxDB-Flask\nextension:\n```\nINFLUXDB_V2_URL                       InfluxDB server API url. Default is <http://localhost:8086>\nINFLUXDB_V2_ORG                       Organization name (used as a default in query and write API). Default is None\nINFLUXDB_V2_TOKEN                     Authentication token.\nINFLUXDB_V2_TIMEOUT                   HTTP client timeout setting for a request specified in milliseconds. Default is 10s.\nINFLUXDB_V2_VERIFY_SSL                Set this to false to skip verifying SSL certificate when calling API from https server. Default is False.\nINFLUXDB_V2_SSL_CA_CERT               Set this to customize the certificate file to verify the peer. Default is None.\nINFLUXDB_V2_CONNECTION_POOL_MAXSIZE   Number of connections to save that can be reused by urllib3. Default is 10.\nINFLUXDB_V2_AUTH_BASIC                Set this to true to enable basic authentication when talking to a InfluxDB 1.8.x that does not use auth-enabled but is protected by \n                                      a reverse proxy with basic authentication. Default is False.\n```',
    'author': 'simon.islam',
    'author_email': 'mail@simonislam.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/simon7426/InfluxDB-Flask',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
